@@ -1,9 +1,8 @@
-my $RCS_Id = '$Id$ ';
+#! perl
 
 package main;
 
 our $dbh;
-our $spp;
 our $config;
 
 package EB::Booking::Decode;
@@ -11,8 +10,8 @@ package EB::Booking::Decode;
 # Author          : Johan Vromans
 # Created On      : Tue Sep 20 15:16:31 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Wed Dec 13 21:36:13 2006
-# Update Count    : 164
+# Last Modified On: Tue Apr 24 08:39:59 2018
+# Update Count    : 198
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -25,7 +24,7 @@ use EB::Format;
 use EB::Booking;		# for norm_btw()
 
 sub new {
-    return bless {};
+    return bless {}, shift;
 }
 
 my @bsr_types =
@@ -55,9 +54,10 @@ sub decode {
 
     $bsk = $dbh->bskid($bsk);
 
-    my $rr = $dbh->do("SELECT bsk_id, bsk_nr, bsk_desc, ".
+    my $rr = $dbh->do("SELECT bsk_id, bsk_nr, bsk_ref, bsk_desc, ".
 		      "bsk_dbk_id, bsk_date, bsk_amount, bsk_saldo, bsk_isaldo, bsk_bky ".
 		      ($dbver lt "001000002" ? ", bsk_paid" : ", bsk_open").
+		      ",bsk_att".
 		      " FROM Boekstukken".
 		      " WHERE bsk_id = ?", $bsk);
 
@@ -66,8 +66,9 @@ sub decode {
 	return;
     }
 
-    my ($bsk_id, $bsk_nr, $bsk_desc, $bsk_dbk_id,
-	$bsk_date, $bsk_amount, $bsk_saldo, $bsk_isaldo, $bsk_bky, $bsk_open) = @$rr;
+    my ($bsk_id, $bsk_nr, $bsk_ref, $bsk_desc, $bsk_dbk_id,
+	$bsk_date, $bsk_amount, $bsk_saldo, $bsk_isaldo, $bsk_bky,
+	$bsk_open, $bsk_att) = @$rr;
 
     my $tot = 0;
     my ($dbktype, $acct, $dbk_desc) = @{$dbh->do("SELECT dbk_type, dbk_acc_id, dbk_desc".
@@ -82,6 +83,8 @@ sub decode {
 	    $cmd =~ s/[^[:alnum:]]/_/g;
 	    $cmd .= ":$bsk_bky" if $ex_bky;
 	    $cmd .= ":$bsk_nr" if $ex_bsknr;
+#	    $cmd .= " --".__xt("cmo:boeking:ref")."=" . _quote($bsk_ref) if defined $bsk_ref;
+	    $rel_code .= ":" . $bsk_ref if defined $bsk_ref;
 	    $cmd .= " ".datefmt_full($bsk_date)." ";
 	    if ( $dbktype == DBKTYPE_INKOOP || $dbktype == DBKTYPE_VERKOOP ) {
 		$cmd .= $no_ivbskdesc ? _quote($rel_code) : _quote($bsk_desc, $rel_code);
@@ -90,17 +93,25 @@ sub decode {
 		$cmd .= _quote($bsk_desc);
 	    }
 	    else {
-		$cmd .= " --totaal=" . numfmt_plain($dbktype == DBKTYPE_INKOOP ? 0-$bsk_amount : $bsk_amount)
+		$cmd .= " --".__xt("cmo:boeking:totaal")."=" . numfmt_plain($dbktype == DBKTYPE_INKOOP ? 0-$bsk_amount : $bsk_amount)
 		  if $ex_tot && $acct;
 	    }
-	    $cmd .= " --beginsaldo=" . numfmt_plain($bsk_isaldo) if $ex_tot && defined $bsk_isaldo;
-	    $cmd .= " --saldo=" . numfmt_plain($bsk_saldo) if $ex_tot && defined $bsk_saldo;
+	    $cmd .= " --".__xt("cmo:boeking:beginsaldo")."=" . numfmt_plain($bsk_isaldo) if $ex_tot && defined $bsk_isaldo;
+	    $cmd .= " --".__xt("cmo:boeking:saldo")."=" . numfmt_plain($bsk_saldo) if $ex_tot && defined $bsk_saldo;
+	    if ( $bsk_att ) {
+		my ( $name, $enc, $contents ) = @{ $dbh->do("SELECT att_name, att_encoding".
+							   " FROM Attachments".
+							   " WHERE att_id = ?", $bsk_att) };
+		$name = sprintf("int://%08d/%s", $bsk_att, $name)
+		  unless $enc == ATTENCODING_URI;
+		$cmd .= " --bijlage=" . _quote($name);
+	    }
 	}
 	else {
 	    $cmd = "Boekstuk $bsk_id, nr $bsk_nr, dagboek " .
 	      $dbh->lookup($bsk_dbk_id, qw(Dagboeken dbk_id dbk_desc =)).
 		"($bsk_dbk_id)".
-		  ", ".datefmt_full(datum $bsk_date).
+		  ", datum ".datefmt_full($bsk_date).
 		    ", ";
 	    if ( $dbktype == DBKTYPE_INKOOP || $dbktype == DBKTYPE_VERKOOP ) {
 		my ($rd, $rt) = @{$dbh->do("SELECT rel_desc,rel_debcrd".
@@ -117,13 +128,14 @@ sub decode {
 	    elsif ( defined $bsk_open ) {
 		$cmd .= $bsk_open ? ", @{[numfmt_plain(abs($bsk_open))]} open" : ", voldaan"
 	    }
+	    $cmd .= ", ref " . _quote($bsk_ref) if defined $bsk_ref;
 	    $cmd .= "\n";
 	}
     };
 
     my $sth = $dbh->sql_exec("SELECT bsr_nr, bsr_date, ".
 			     "bsr_desc, bsr_amount, bsr_btw_id, bsr_btw_class, ".
-			     "bsr_type, bsr_acc_id, bsr_rel_code, bsr_paid ".
+			     "bsr_type, bsr_acc_id, bsr_rel_code, bsr_paid, bsr_ref ".
 			     " FROM Boekstukregels".
 			     " WHERE bsr_bsk_id = ?".
 			     " ORDER BY bsr_nr", $bsk);
@@ -137,7 +149,7 @@ sub decode {
 
     while ( $rr ) {
 	my ($bsr_nr, $bsr_date, $bsr_desc, $bsr_amount, $bsr_btw_id,
-	    $bsr_btw_class, $bsr_type, $bsr_acc_id, $bsr_rel_code, $bsr_paid) = @$rr;
+	    $bsr_btw_class, $bsr_type, $bsr_acc_id, $bsr_rel_code, $bsr_paid, $bsr_ref) = @$rr;
 	if ( $bsr_nr == 1) {
 	    $setup->($bsr_rel_code);
 	}
@@ -170,19 +182,29 @@ sub decode {
 	my $a = EB::Booking::->norm_btw($bsr_amount, $bsr_btw_id);
 	$tot += $a->[0];
 
-	next unless $trail;
+	$rr = $sth->fetchrow_arrayref, next unless $trail;
 
 	$bsr_acc_id ||= "";
 	my $btw = "";
 
 	# Refactor later.
 	if ( $bsr_btw_class & BTWKLASSE_BTW_BIT ) {
+	    my $alias = $bsr_btw_id;
+	    if ( $bsr_btw_id > BTW_CODE_AUTO ) {
+		if ( $bsr_btw_id & 1 ) {
+		    $alias = $dbh->lookup($bsr_btw_id, qw(BTWTabel btw_id btw_desc));
+		}
+		else {
+		    $alias = $dbh->lookup($bsr_btw_id, qw(BTWTabel btw_id btw_alias));
+		    $alias .= qw( - + )[$dbh->lookup($bsr_btw_id, qw(BTWTabel btw_id btw_incl))];
+		}
+	    }
 	    my $ko = $bsr_btw_class & BTWKLASSE_KO_BIT ? 1 : 0;
 	    if ( $ex_btw ) {
-		$btw = $bsr_btw_id . qw(O K)[$ko];
+		$btw = $alias . qw(O K)[$ko];
 	    }
 	    else {
-		$btw .= $bsr_btw_id
+		$btw .= $alias
 		  if btw_code($bsr_acc_id) != $bsr_btw_id
 		    || ($bsr_type == 0 && $dbktype == DBKTYPE_MEMORIAAL);
 		$btw .= qw(O K)[$ko]
@@ -212,25 +234,36 @@ sub decode {
 		|| $dbktype == DBKTYPE_MEMORIAAL ) {
 	    $bsr_amount = -$bsr_amount;
 	    my $dd = "";
-	    $dd = " ".datefmt_full($bsr_date) unless $bsr_date eq $bsk_date;
+	    # Explicitly add the date if it is different, or when the
+	    # description could be parsed as a date (bug #40).
+	    $dd = " ".datefmt_full($bsr_date)
+	      if $bsr_date ne $bsk_date || $bsr_desc =~ /^[[:digit:]]+-/;
+
 	    if ( $bsr_type == 0 ) {
 		$cmd .= $single ? " " : " \\\n\t";
 		$cmd .= "std$dd " . _quote($bsr_desc) . " " .
 		  numfmt_plain($bsr_amount) . $btw . " " .
 		    $bsr_acc_id;
 	    }
+	    elsif ( $bsr_ref && ( $bsr_type == 1 || $bsr_type == 2 ) ) {
+		my $type = $bsr_type == 1 ? "deb" : "crd";
+		$cmd .= $single ? " " : " \\\n\t";
+		$cmd .= "$type$dd " . _quote($bsr_ref) . " " .
+		  numfmt_plain($bsr_amount);
+	    }
 	    elsif ( $bsr_type == 1 || $bsr_type == 2 ) {
 		my $type = $bsr_type == 1 ? "deb" : "crd";
 		$cmd .= $single ? " " : " \\\n\t";
 
 		# Check for a full payment.
-		my $sth = $dbh->sql_exec("SELECT bsk_amount, dbk_desc, bsk_nr, bsk_bky".
-					 " FROM Boekstukken, Dagboeken".
+		my $sth = $dbh->sql_exec("SELECT bsk_amount, dbk_desc, bsk_nr, bsk_ref, bsr_rel_code, bsk_bky".
+					 " FROM Boekstukken, Boekstukregels, Dagboeken".
 					 " WHERE bsk_dbk_id = dbk_id".
+					 " AND bsr_bsk_id = bsk_id".
 					 " AND bsk_id = ?", $bsr_paid);
-		my ($paid, $dbk, $nr, $bky) = @{$sth->fetchrow_arrayref};
+		my ($paid, $dbk, $nr, $ref, $rel, $bky) = @{$sth->fetchrow_arrayref};
 		$sth->finish;
-		if ( $paid == $bsr_amount) {
+		if ( $paid == $bsr_amount ) {
 		    # Matches -> Full payment
 		    $cmd .= "$type$dd " . _quote($bsr_rel_code) . " " .
 		      numfmt_plain($bsr_amount);
@@ -239,9 +272,18 @@ sub decode {
 		    # Partial payment. Use boekstuknummer.
 		    $dbk = lc($dbk);
 		    $dbk =~ s/[^[:alnum:]]/_/g;
-		    $cmd .= "$type$dd $dbk";
-		    $cmd .= ":$bky" if ($opts->{boekjaar}||$opts->{d_boekjaar}) ne $bky;
-		    $cmd .= ":$nr " . numfmt_plain($bsr_amount);
+		    my $t;
+		    if ( defined $ref ) {
+			$t = $rel;
+			$t .= ":$bky" if ($opts->{boekjaar}||$opts->{d_boekjaar}) ne $bky;
+			$t .= ":$ref";
+		    }
+		    else {
+			$t = $dbk;
+			$t .= ":$bky" if ($opts->{boekjaar}||$opts->{d_boekjaar}) ne $bky;
+			$t .= ":$nr";
+		    }
+		    $cmd .= join(" ", $type.$dd, _quote($t), numfmt_plain($bsr_amount));
 		}
 	    }
 	}
