@@ -1,4 +1,6 @@
-#!perl
+#! perl --			-*- coding: utf-8 -*-
+
+use utf8;
 
 package main;
 
@@ -7,12 +9,11 @@ our $dbh;
 
 package EB::Booking::IV;
 
-# RCS Id          : $Id: IV.pm,v 1.54 2008/03/02 15:04:53 jv Exp $
 # Author          : Johan Vromans
 # Created On      : Thu Jul  7 14:50:41 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Sun Mar  2 16:01:17 2008
-# Update Count    : 302
+# Last Modified On: Tue Apr 24 12:25:35 2018
+# Update Count    : 371
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -20,8 +21,6 @@ package EB::Booking::IV;
 use strict;
 use warnings;
 
-our $VERSION = sprintf "%d.%03d", q$Revision: 1.54 $ =~ /(\d+)/g;
-
 # Dagboek type 1: Inkoop
 # Dagboek type 2: Verkoop
 
@@ -40,10 +39,10 @@ sub perform {
     my $dagboek = $opts->{dagboek};
     my $dagboek_type = $opts->{dagboek_type};
     my $bsk_ref = $opts->{ref};
+    my $bsk_att = $opts->{bijlage};
 
-    if ( defined $bsk_ref && $bsk_ref =~ /^\d+$/ ) {
-	warn("?".__x("Boekingsreferentie moet tenminste n niet-numeriek teken bevatten: {ref}", ref => $bsk_ref)."\n");
-	return;
+    if ( defined $bsk_att ) {
+	return unless $self->check_attachment($bsk_att);
     }
 
     unless ( $dagboek_type == DBKTYPE_INKOOP || $dagboek_type == DBKTYPE_VERKOOP) {
@@ -95,15 +94,21 @@ sub perform {
 
     if ( $cfg->val(qw(general ivdesc), undef) ) {
 	$gdesc  = shift(@$args);
-	$debcode = shift(@$args);
+	my $arg = shift(@$args);
+	( $debcode, $bsk_ref ) = relref($arg, $bsk_ref);
+	if ( defined $bsk_ref && $bsk_ref =~ /^\d+$/ ) {
+	    warn("?".__x("Boekingsreferentie moet tenminste één niet-numeriek teken bevatten: {ref}", ref => $bsk_ref)."\n");
+	    return;
+	}
+
 	$rr = $dbh->do("SELECT rel_code, rel_acc_id, rel_btw_status FROM Relaties" .
 		       " WHERE UPPER(rel_code) = ?" .
 		       "  AND " . ($iv ? "NOT " : "") . "rel_debcrd" .
 		       "  AND rel_ledger = ?",
 		       uc($debcode), $dagboek);
 	unless ( defined($rr) ) {
-	    unshift(@$args, $debcode);
-	    $debcode = $gdesc;
+	    unshift(@$args, $arg);
+	    ( $debcode, $bsk_ref ) = relref($gdesc, $bsk_ref);
 	    $gdesc = "";
 	    $rr = $dbh->do("SELECT rel_code, rel_acc_id, rel_btw_status FROM Relaties" .
 			   " WHERE UPPER(rel_code) = ?" .
@@ -119,15 +124,21 @@ sub perform {
 	}
     }
     else {
-	$debcode = shift(@$args);
+	my $arg = shift(@$args);
+	( $debcode, $bsk_ref ) = relref($arg, $bsk_ref);
+	if ( defined $bsk_ref && $bsk_ref =~ /^\d+$/ ) {
+	    warn("?".__x("Boekingsreferentie moet tenminste één niet-numeriek teken bevatten: {ref}", ref => $bsk_ref)."\n");
+	    return;
+	}
+
 	$rr = $dbh->do("SELECT rel_code, rel_acc_id, rel_btw_status FROM Relaties" .
 		       " WHERE UPPER(rel_code) = ?" .
 		       "  AND " . ($iv ? "NOT " : "") . "rel_debcrd" .
 		       "  AND rel_ledger = ?",
 		       uc($debcode), $dagboek);
 	unless ( defined($rr) ) {
-	    $gdesc = $debcode;
-	    $debcode = shift(@$args);
+	    $gdesc = $arg;
+	    ( $debcode, $bsk_ref ) = relref(shift(@$args), $bsk_ref);
 	    $rr = $dbh->do("SELECT rel_code, rel_acc_id, rel_btw_status FROM Relaties" .
 			   " WHERE UPPER(rel_code) = ?" .
 			   "  AND " . ($iv ? "NOT " : "") . "rel_debcrd" .
@@ -145,6 +156,7 @@ sub perform {
     my ($rel_acc_id, $rel_btw);
     ($debcode, $rel_acc_id, $rel_btw) = @$rr;
 
+    my $btw_adapt = $cfg->val(qw(strategy btw_adapt), 0);
     my $nr = 1;
     my $bsk_id;
     my $bsk_nr;
@@ -154,6 +166,7 @@ sub perform {
 	return "?"._T("Deze opdracht is onvolledig. Gebruik de \"help\" opdracht voor meer aanwijzingen.")."\n"
 	  unless @$args >= 2;
 	my ($desc, $amt, $acct) = splice(@$args, 0, 3);
+	my $bsr_ref;
 	$desc = $gdesc if $desc !~ /\S/;
 	$gdesc = $desc if $gdesc !~ /\S/;
 	$acct ||= $rel_acc_id;
@@ -221,7 +234,8 @@ sub perform {
 	}
 
 	# Amount can override BTW id with @X postfix.
-	my ($namt, $btw_spec) = $does_btw ? $self->amount_with_btw($amt, $btw_id) : amount($amt);
+	my ($namt, $btw_spec, $btw_explicit) =
+	  $does_btw ? $self->amount_with_btw($amt, $btw_id) : amount($amt);
 	unless ( defined($namt) ) {
 	    warn("?".__x("Ongeldig bedrag: {amt}", amt => $amt)."\n");
 	    return;
@@ -256,14 +270,78 @@ sub perform {
 	    }
 	    # Void BTW voor non-EU en verlegd.
 	    if ( $btw_id && ($rel_btw == BTWTYPE_NORMAAL || $rel_btw == BTWTYPE_INTRA) ) {
-		my $tg = $dbh->lookup($btw_id, qw(BTWTabel btw_id btw_tariefgroep));
-		unless ( defined($tg) ) {
+
+		my $res = $dbh->do( "SELECT btw_tariefgroep, btw_start, btw_end, btw_alias, btw_desc, btw_incl".
+				    " FROM BTWTabel".
+				    " WHERE btw_id = ?",
+				    $btw_id );
+		my $incl = $res->[5];
+
+		if ( $incl && $rel_btw == BTWTYPE_INTRA ) {
+		    if ( $btw_explicit ) {	# user specified -> warning
+			warn("!".__x("BTW code {code} is inclusief BTW maar relatie {rel} is intra-communautair",
+				     code => $btw_id, rel => $debcode)."\n" );
+		    }
+		    warn("!".__x("Er wordt geen BTW berekend voor intra-relatie {rel}",
+				     rel => $debcode)."\n" );
+		}
+
+		my $tg;
+		unless ( defined($res) && defined( $tg = $res->[0] ) ) {
 		    warn("?".__x("Onbekende BTW-code: {code}", code => $btw_id)."\n");
 		    return;
 		}
-		my $t = "btw_" . ($iv ? "i" : "v");
-		$t .= $tg == BTWTARIEF_HOOG ? 'h' : 'l';
-		$btw_acc = $dbh->std_acc($t);
+		if ( defined( $res->[1] ) && $res->[1] gt $date ) {
+		    my $ok = 0;
+		    if ( $btw_adapt && !$btw_explicit ) {
+			my $rr = $dbh->do( "SELECT btw_id, btw_desc".
+					   " FROM BTWTabel".
+					   " WHERE btw_tariefgroep = ?".
+					   " AND btw_end >= ?".
+					   " AND " . ( $incl ? "" : "NOT " ) . "btw_incl".
+					   " ORDER BY btw_id",
+					   $tg, $date );
+			if ( $rr && $rr->[0] ) {
+			    warn("%".__x("BTW-code: {code} aangepast naar {new} i.v.m. de boekingsdatum",
+					 code => $res->[3]||$res->[4]||$btw_id,
+					 new => $rr->[1]||$rr->[0],
+					)."\n");
+			    $btw_id = $rr->[0];
+			    $ok++;
+			}
+		    }
+		    unless ( $ok ) {
+			warn("!".__x("BTW-code: {code} is nog niet geldig op de boekingsdatum",
+				     code => $res->[3]||$res->[4]||$btw_id)."\n");
+		    }
+		}
+		if ( defined( $res->[2] ) && $res->[2] lt $date ) {
+		    my $ok = 0;
+		    if ( $btw_adapt && !$btw_explicit ) {
+			my $rr = $dbh->do( "SELECT btw_id, btw_desc".
+					   " FROM BTWTabel".
+					   " WHERE btw_tariefgroep = ?".
+					   " AND btw_start <= ?".
+					   " AND " . ( $incl ? "" : "NOT " ) . "btw_incl".
+					   " ORDER BY btw_id",
+					   $tg, $date );
+			if ( $rr && $rr->[0] ) {
+			    warn("%".__x("BTW-code: {code} aangepast naar {new} i.v.m. de boekingsdatum",
+					 code => $res->[3]||$res->[4]||$btw_id,
+					 new => $rr->[1]||$rr->[0],
+					)."\n");
+			    $btw_id = $rr->[0];
+			    $ok++;
+			}
+		    }
+		    unless ( $ok ) {
+			warn("!".__x("BTW-code: {code} is niet meer geldig op de boekingsdatum",
+				     code => $res->[3]||$res->[4]||$btw_id)."\n");
+		    }
+		}
+		my $tp = BTWTARIEVEN->[$tg];
+		my $t = qw(v i)[$iv] . lc(substr($tp, 0, 1));
+		$btw_acc = $dbh->std_acc("btw_$t");
 	    }
 	}
 	elsif ( $btw_id ) {
@@ -275,11 +353,11 @@ sub perform {
 	$dbh->sql_insert("Boekstukregels",
 			 [qw(bsr_nr bsr_date bsr_bsk_id bsr_desc bsr_amount
 			     bsr_btw_id bsr_btw_acc bsr_btw_class bsr_type bsr_acc_id
-			     bsr_rel_code bsr_dbk_id)],
+			     bsr_rel_code bsr_dbk_id bsr_ref)],
 			 $nr++, $date, $bsk_id, $desc, $amt,
 			 $btw_id, $btw_acc,
 			 BTWKLASSE($does_btw ? defined($kstomz) : 0, $rel_btw, defined($kstomz) ? $kstomz : $iv),
-			 0, $acct, $debcode, $dagboek);
+			 0, $acct, $debcode, $dagboek, $bsr_ref);
     }
 
     my $ret = $self->journalise($bsk_id, $iv, $totaal);
@@ -292,7 +370,7 @@ sub perform {
 #	warn("update $ac with ".numfmt($amt)."\n") if $trace_updates;
 #	$dbh->upd_account($ac, $amt);
 #    }
-    my $tot = $ret->[$#{$ret}]->[6];
+    my $tot = $ret->[$#{$ret}]->[8]; # ERROR PRONE
     $dbh->sql_exec("UPDATE Boekstukken SET bsk_amount = ?, bsk_open = ? WHERE bsk_id = ?",
 		   $tot, $tot, $bsk_id)->finish;
 
@@ -301,7 +379,7 @@ sub perform {
     $tot = -$tot if $iv;
     my $fail = defined($totaal) && $tot != $totaal;
     if ( $opts->{journal} ) {
-	warn("?"._T("Dit overicht is ter referentie, de boeking is niet uitgevoerd!")."\n") if $fail;
+	warn("?"._T("Dit overzicht is ter referentie, de boeking is niet uitgevoerd!")."\n") if $fail;
 	EB::Report::Journal->new->journal
 	    ({select => $bsk_id,
 	      d_boekjaar => $bky,
@@ -310,15 +388,14 @@ sub perform {
 
     if ( $fail ) {
 	$dbh->rollback;
-	return "?"._T("Boeking ".
-		      join(":", $dbh->lookup($dagboek, qw(Dagboeken dbk_id dbk_desc)), $bsk_nr).
-		      " is niet uitgevoerd!")." ".
+	return "?".__x("Boeking {bk} is niet uitgevoerd!",
+		       bk => join(":", $dbh->lookup($dagboek, qw(Dagboeken dbk_id dbk_desc)), $bsk_nr))." ".
 	  __x(" Boekstuk totaal is {act} in plaats van {exp}",
 	      act => numfmt($tot), exp => numfmt($totaal)) . ".";
     }
-    else {
-	$dbh->commit;
-    }
+
+    $self->add_attachment( $bsk_att, $bsk_id ) if $bsk_att;
+    $dbh->commit;
 
     # TODO -- need this to get a current booking.
     $opts->{verbose} || 1
@@ -326,4 +403,13 @@ sub perform {
 	: "";
 }
 
+sub relref {
+    my ( $rel, $ref ) = @_;
+    return ( $rel, $ref ) if defined $ref;
+    if ( $rel =~ /^(.+):(.+)/ ) {
+	( $rel, $ref ) = ( $1, $2 );
+    }
+    return ( $rel, $ref );
+}
+
 1;
