@@ -5,12 +5,11 @@ package main;
 our $dbh;
 
 # Report.pm -- Report tools
-# RCS Info        : $Id: Report.pm,v 1.7 2008/02/07 12:13:27 jv Exp $
 # Author          : Johan Vromans
 # Created On      : Mon Nov 14 21:46:04 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Thu Feb  7 13:13:25 2008
-# Update Count    : 42
+# Last Modified On: Tue Nov  1 10:53:12 2011
+# Update Count    : 45
 # Status          : Unknown, Use with caution!
 
 package EB::Report;
@@ -18,8 +17,6 @@ package EB::Report;
 use strict;
 use warnings;
 
-our $VERSION = sprintf "%d.%03d", q$Revision: 1.7 $ =~ /(\d+)/g;
-
 use EB;
 use EB::Format qw(numfmt);
 use EB::Report::GenBase;
@@ -35,9 +32,9 @@ sub GetTAccountsBal {
     # balans(r, t0)
     $dbh->sql_exec("DELETE FROM TAccounts");
     $dbh->sql_exec("INSERT INTO TAccounts".
-		   " (acc_id,acc_desc,acc_balres,acc_debcrd,".
+		   " (acc_id,acc_desc,acc_balres,acc_debcrd,acc_dcfixed,".
 		   "acc_ibalance,acc_balance,acc_struct)".
-		   " SELECT acc_id,acc_desc,acc_balres,acc_debcrd,".
+		   " SELECT acc_id,acc_desc,acc_balres,acc_debcrd,acc_dcfixed,".
 		   "acc_ibalance,acc_ibalance AS acc_balance,acc_struct".
 		   " FROM Accounts")->finish;
 
@@ -140,6 +137,10 @@ sub GetTAccountsRes {
 		       $acc_balance, $acc_id)->finish;
     }
 
+    # zet eindsaldo op beginsaldo
+    $dbh->sql_exec("UPDATE TAccounts".
+           " SET acc_balance = acc_ibalance")->finish;
+
     # eindsaldo(r, t2) = beginsaldo(r, t1) + sum(journaal, r, t1..t2)
     $sth = $dbh->sql_exec("SELECT jnl_acc_id,SUM(jnl_amount)".
 			     " FROM Journal,TAccounts".
