@@ -1,11 +1,12 @@
-#! perl
+#! perl --			-*- coding: utf-8 -*-
+
+use utf8;
 
-# RCS Id          : $Id: Opening.pm,v 1.41 2008/07/19 16:49:56 jv Exp $
 # Author          : Johan Vromans
 # Created On      : Tue Aug 30 09:49:11 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Thu May  1 17:55:43 2008
-# Update Count    : 290
+# Last Modified On: Mon Oct 23 20:20:12 2017
+# Update Count    : 318
 # Status          : Unknown, Use with caution!
 
 package main;
@@ -18,8 +19,6 @@ package EB::Tools::Opening;
 use strict;
 use warnings;
 
-our $VERSION = sprintf "%d.%03d", q$Revision: 1.41 $ =~ /(\d+)/g;
-
 use EB;
 use EB::Format;
 
@@ -128,7 +127,7 @@ sub set_balans {
 	$anew = -$anew if ($e->[0] xor $debcrd);
 	$self->{o}->{balans}->{$acct}->[1] += $anew;
     }
-    else {
+    elsif ( $anew ) {		# skip nul values
 	$self->{o}->{balans}->{$acct} = [ $debcrd, $anew ];
     }
     "";
@@ -166,7 +165,7 @@ sub set_relatie {
     return __x("Ongeldige datum: {date}", date => $date)."\n"
       unless $t;
     $date = $t;
-    return __x("Datum {date} valt niet vr het boekjaar",
+    return __x("Datum {date} valt niet vóór het boekjaar",
 	       date => datefmt_full($date))."\n"
       if $self->{o}->{begindatum} && $self->{o}->{begindatum} le $t;
     $bky = substr($date, 0, 4) unless defined $bky;
@@ -210,7 +209,7 @@ sub set_relatie {
     return __x("Ongeldig bedrag: {amount}", amount => $amt)."\n" unless defined($anew = amount($amt));
 
     $self->check_open(0);
-    push(@{$self->{o}->{relatie}}, [$bky, $nr, $date, $desc, $type, $code, $acct, $anew]);
+    push(@{$self->{o}->{relatie}}, [$bky, $nr, $date, $desc, $type, $code, $dbk, $acct, $anew]);
     "";
 }
 
@@ -314,13 +313,13 @@ sub open {
 			 rcrd => numfmt(-$rcredit)).
 		     ($rdebet == -$rcredit
 		      ? __x(" (balanstotaal {total})", total => numfmt($o->{balanstotaal} - $rdebet))
-		      : ())."\n");
+		      : '')."\n");
 		warn(__x("Boekhoudkundig residu debet = {rdeb}, credit = {rcrd}",
 			 rdeb => numfmt($bdebet),
 			 rcrd => numfmt(-$bcredit)).
 		     ($bdebet == -$bcredit
 		      ? __x(" (balanstotaal {total})", total => numfmt($o->{balanstotaal} - $bdebet))
-		      : ())."\n");
+		      : '')."\n");
 	    }
 
 	    # Helpful hints...
@@ -331,7 +330,7 @@ sub open {
 
 	    # Process relations.
 	    foreach my $r ( @{$o->{relatie}} ) {
-		my ($bky, $nr, $date, $desc, $debcrd, $code, $acct, $amt) = @$r;
+		my ($bky, $nr, $date, $desc, $debcrd, $code, $dbk, $acct, $amt) = @$r;
 
 		if ( $debcrd ) {
 		    $adeb = $acct;
@@ -457,13 +456,13 @@ sub open {
 	my $dbk_inkoop;
 	my $dbk_verkoop;
 	foreach my $r ( @{$o->{relatie}} ) {
-	    my ($bky, $nr, $date, $desc, $debcrd, $code, $acct, $amt) = @$r;
+	    my ($bky, $nr, $date, $desc, $debcrd, $code, $dagboek, $acct, $amt) = @$r;
 	    $nr = $dbh->get_sequence("bsk_nr_0_seq") unless defined $nr;
 
-	    my ($dagboek) = @{$dbh->do("SELECT dbk_id".
-				       " FROM Dagboeken".
-				       " WHERE dbk_acc_id = ?",
-				       $acct)};
+#	    my ($dagboek) = @{$dbh->do("SELECT dbk_id".
+#				       " FROM Dagboeken".
+#				       " WHERE dbk_acc_id = ?",
+#				       $acct)};
 
 	    if ( $debcrd ) {
 #		unless ( $dbk_verkoop ) {
@@ -524,7 +523,9 @@ sub reopen {
 
     # New begin date is old + one year.
     my $y = parse_date($dbh->adm("end"), undef, 1);
-    if ( $y gt iso8601date() ) {
+
+    # Avoid opening a new book year too soon.
+    if ( $y gt parse_date(iso8601date(), undef, 0, 6) ) {
 	warn(__x("Begindatum {year} komt in de toekomst te liggen",
 		 year => $y)."\n");
 	$fail++;
@@ -577,64 +578,82 @@ sub reopen {
 
 sub shellhelp {
     my ($self, $cmd) = @_;
-    <<EOS;
-Het openen van een administratie kan slechts nmaal gebeuren, vr
+    _T( <<EOS );
+Het openen van een administratie kan slechts éénmaal gebeuren, vóór
 het invoeren van de eerste mutatie. Het openen van een nieuw boekjaar
-kan te allen tijde worden uitgevoerd, uiteraard maar n keer per
+kan te allen tijde worden uitgevoerd, uiteraard maar één keer per
 boekjaar.
 
 Het openen kan een aantal opdrachten omvatten, en wordt afgesloten met
 de opdracht "adm_open". Zolang deze laatste opdracht niet is gegeven
 blijft de administratie ongewijzigd. Alle benodigde opdrachten moeten
-dan ook in n enkele EekBoek shell sessie worden afgehandeld.
+dan ook in één enkele EekBoek shell sessie worden afgehandeld.
 
 Mogelijke opdrachten voor openen van een boekjaar:
 
   adm_btwperiode [ jaar | kwartaal | maand ]
+
   adm_boekjaarcode <code>
-                Een code van max 4 letters en/of cijfers waarmee het
-		boekjaar kan worden gedentificeerd.
-		Standaard wordt het jaartal van het boekjaar genomen.
+
+Een code van max 4 letters en/of cijfers waarmee het
+boekjaar kan worden geïdentificeerd.
+Standaard wordt het jaartal van het boekjaar genomen.
+
   adm_open
-		Alle informatie die met de bovenstaande opdrachten is
-		ingevoerd, wordt verwerkt.
+
+Alle informatie die met de bovenstaande opdrachten is
+ingevoerd, wordt verwerkt.
 
 Opdrachten voor het openen van een administratie:
 
   adm_naam "Naam van de administratie"
+
   adm_btwperiode [ jaar | kwartaal | maand ]
+
   adm_begindatum <jaar>
-		Een administratie loopt altijd van 1 januari tot en
-		met 31 december van een kalenderjaar.
+
+Een administratie loopt altijd van 1 januari tot en
+met 31 december van een kalenderjaar.
+
   adm_boekjaarcode <code>
-                Een code van max 4 letters en/of cijfers waarmee het
-		boekjaar kan worden gedentificeerd.
-		Standaard wordt het jaartal van het boekjaar genomen.
-		De boekjaarcode is alleen relevant indien er meerdere
-		boekjaren in n administratie worden bijgehouden.
+
+Een code van max 4 letters en/of cijfers waarmee het
+boekjaar kan worden geïdentificeerd.
+
+Standaard wordt het jaartal van het boekjaar genomen.
+De boekjaarcode is alleen relevant indien er meerdere
+boekjaren in één administratie worden bijgehouden.
+
   adm_balanstotaal <bedrag>
-		Als een balanstotaal is opgegeven, moeten er ook
-		openingsbalansboekingen worden uitgevoerd met een of
-                meer adm_balans opdrachten.
+
+Als een balanstotaal is opgegeven, moeten er ook
+openingsbalansboekingen worden uitgevoerd met een of
+meer adm_balans opdrachten.
+
   adm_balans <balansrekening> <bedrag>
-		De debet en credit boekingen moeten uiteindelijk
-		allebei gelijk zijn aan het opgegeven balanstotaal.
-		Indien er een bedrag is opgegeven voor de balansrekening
-		Crediteuren of Debiteuren, dan moet er voor dit bedrag
-		ook openstaande posten worden ingevoerd met een of
-		meer adm_relatie opdrachten.
+
+De debet en credit boekingen moeten uiteindelijk
+allebei gelijk zijn aan het opgegeven balanstotaal.
+Indien er een bedrag is opgegeven voor de balansrekening
+Crediteuren of Debiteuren, dan moet er voor dit bedrag
+ook openstaande posten worden ingevoerd met een of
+meer adm_relatie opdrachten.
+
   adm_relatie <boekstuk> <datum> <code> <omschrijving> <bedrag>
-		Invoeren van een openstaande post uit het voorgaande
-		boekjaar. Het <boekstuk> moet volledig zijn, dus
-                <dagboek>:<boekjaar>:<nummer>.
+
+Invoeren van een openstaande post uit het voorgaande
+boekjaar. Het <boekstuk> moet volledig zijn, dus
+<dagboek>:<boekjaar>:<nummer>.
+
   adm_open
-		Alle informatie die met de bovenstaande opdrachten is
-		ingevoerd, wordt verwerkt.
+
+Alle informatie die met de bovenstaande opdrachten is
+ingevoerd, wordt verwerkt.
 EOS
 }
 
 sub help_btwperiode {
-    <<EOS;
+    _T( <<EOS );
 Specifeer de BTW aangifteperiode voor het nieuw te openen jaar.
 
   adm_btwperiode [ jaar | kwartaal | maand ]
@@ -645,14 +664,14 @@ EOS
 }
 
 sub help_boekjaarcode {
-    <<EOS;
+    _T( <<EOS );
 Specificeert de boekjaarcode voor het nieuw te openen jaar.
 
   adm_boekjaarcode <code>
 
 De boekjaarcode telt maximaal 4 letters en/of cijfers.
 Standaard wordt het jaartal van het te openen boekjaar genomen.
-De boekjaarcode is alleen belangrijk indien er meerdere boekjaren in n
+De boekjaarcode is alleen belangrijk indien er meerdere boekjaren in één
 administratie worden bijgehouden.
 
 Deze opdracht kan worden gebruikt bij het openen van een boekjaar.
@@ -661,8 +680,8 @@ EOS
 }
 
 sub help_naam {
-    <<EOS;
-Specificeert de naam van de administatie, te gebruiken voor rapportages.
+    _T( <<EOS );
+Specificeert de naam van de administratie, te gebruiken voor rapportages.
 
   adm_naam "Naam van de administratie"
 
@@ -672,7 +691,7 @@ EOS
 }
 
 sub help_begindatum {
-    <<EOS;
+    _T( <<EOS );
 Specificeert de begindatum van de administratie.
 Een administratie loopt altijd van 1 januari tot en met 31 december
 van een kalenderjaar. Daarom moet als begindatum een jaartal worden
@@ -686,7 +705,7 @@ EOS
 }
 
 sub help_balanstotaal {
-    <<EOS;
+    _T( <<EOS );
 Specificeert het balanstotaal voor de in te voeren openingbalans.
 
   adm_balanstotaal <bedrag>
@@ -702,7 +721,7 @@ EOS
 }
 
 sub help_balans {
-    <<EOS;
+    _T( <<EOS );
 Specificeert een balanspost voor de openingsbalans.
 
   adm_balans <balansrekening> <bedrag>
@@ -721,7 +740,7 @@ EOS
 }
 
 sub help_relatie {
-    <<EOS;
+    _T( <<EOS );
 Specificeert een openstaande post uit een voorgaand boekjaar.
 
   adm_relatie <boekstuk> <datum> <code> <omschrijving> <bedrag>
