@@ -1,11 +1,17 @@
 #! perl
 
+# Utils.pm -- 
+# Author          : Johan Vromans
+# Created On      : Wed Sep 21 13:09:01 2005
+# Last Modified By: Johan Vromans
+# Last Modified On: Tue May 14 20:21:30 2013
+# Update Count    : 127
+# Status          : Unknown, Use with caution!
+
 package EB::Utils;
 
 use strict;
 
-our $VERSION = sprintf "%d.%03d", q$Revision: 1.8 $ =~ /(\d+)/g;
-
 use base qw(Exporter);
 
 our @EXPORT;
@@ -13,8 +19,36 @@ our @EXPORT_OK;
 
 use Time::Local;
 
+# We're imported by EB that exports _T. Kinda catch-22.
+*_T = *EB::_T;
+
+# These are only used by the BTW Aangifte modules.
+# Note these are translated using _T where appropriate.
+our @months =
+      split(" ", "Jan Feb Mrt Apr Mei Jun Jul Aug Sep Okt Nov Dec");
+our @month_names =
+      split(" ", "Januari Februari Maart April Mei Juni Juli Augustus September Oktober November December");
+our @days =
+      split(" ", "Zon Maa Din Woe Don Vri Zat");
+our @day_names =
+      split(" ", "Zondag Maandag Dinsdag Woensdag Donderdag Vrijdag Zaterdag");
+
+my $_i;
+
 my %rev_months;
+$_i = 1;
+foreach ( @months ) {
+    $rev_months{ lc $_  } = $_i;
+    $rev_months{ "m$_i" } = $_i;
+    $rev_months{ sprintf("m%02d", $_i) } = $_i;
+    $_i++;
+}
+
 my %rev_month_names;
+$_i = 1;
+foreach ( @month_names ) {
+    $rev_month_names{ lc $_ } = $_i++;
+}
 
 sub parse_date {
     my ($date, $default_year, $delta_d, $delta_m, $delta_y) = @_;
@@ -30,22 +64,10 @@ sub parse_date {
 	($d, $m, $y) = ($1, $2, $3);
     }
     elsif ( $date =~ /^(\d\d?)-(\d\d?)$/ ) {
+	return unless $default_year;
 	($d, $m, $y) = ($1, $2, $default_year);
     }
     elsif ( $date =~ /^(\d\d?) (\w+)$/ ) {
-	unless ( %rev_months ) {
-	    my $i = 1;
-	    foreach ( @EB::months ) {
-		$rev_months{lc $_} = $i;
-		$rev_months{"m$i"} = $i;
-		$rev_months{sprintf("m%02d", $i)} = $i;
-		$i++;
-	    }
-	    $i = 1;
-	    foreach ( @EB::month_names ) {
-		$rev_month_names{lc $_} = $i++;
-	    }
-	}
 	return unless $default_year;
 	return unless $m = $rev_month_names{$2} || $rev_months{$2};
 	($d, $y) = ($1, $default_year);
@@ -53,19 +75,32 @@ sub parse_date {
     else {
 	return;		# invalid format
     }
-    $y = $y + $delta_y if $delta_y;
-    $m = $m + $delta_m if $delta_m;
-    $m = 1, $y++ if $m > 12;
-    $m = 12, $y-- if $m < 0;
-    my $time = eval { timelocal(0, 0, 0, $d, $m-1, $y) };
+
+    # The date, as delivered, must be valid.
+    my $time = eval { timelocal(0, 0, 12, $d, $m-1, $y) };
     return unless $time;	# invalid date
+
+    # Handle deltas.
+    $y += $delta_y if $delta_y;
+    $m += $delta_m if $delta_m;
+    while ( $m > 12 ) { $m -= 12, $y++ }
+    while ( $m < 1  ) { $m += 12; $y-- }
+    $delta_d += $d - 1;
+
+    # New date, as of 1st of the month.
+    $time = eval { timelocal(0, 0, 12, 1, $m-1, $y) };
+    return unless $time;	# invalid date
+
+    # Apply delta.
     $time += $delta_d * 24*60*60 if $delta_d;
+
+    # Convert and return.
     my @tm = localtime($time);
     @tm = (1900 + $tm[5], 1 + $tm[4], $tm[3]);
     wantarray ? @tm : sprintf("%04d-%02d-%02d", @tm);
 }
 
-BEGIN { push(@EXPORT, qw(parse_date)) }
+push( @EXPORT, qw(parse_date) );
 
 sub parse_date_range {
     my ($range, $default_year) = @_;
@@ -76,20 +111,6 @@ sub parse_date_range {
     my ($d1, $m1, $y1, $d2, $m2, $y2);
     my $datefix;
 
-    unless ( %rev_months ) {
-	my $i = 1;
-	foreach ( @EB::months ) {
-	    $rev_months{lc $_} = $i;
-	    $rev_months{"m$i"} = $i;
-	    $rev_months{sprintf("m%02d", $i)} = $i;
-	    $i++;
-	}
-	$i = 1;
-	foreach ( @EB::month_names ) {
-	    $rev_month_names{lc $_} = $i++;
-	}
-    }
-
     $range = lc($range);
 
     # 2004-03-04 - 2004-05-06 -> [ "2004-03-04", "2004-05-06" ]
@@ -174,9 +195,9 @@ sub parse_date_range {
 	$m2 = 1, $y2++ if ++$m2 > 12;
     }
 
-    my $time1 = eval { timelocal(0, 0, 0, $d1, $m1-1, $y1) };
+    my $time1 = eval { timelocal(0, 0, 12, $d1, $m1-1, $y1) };
     return unless $time1;	# invalid date
-    my $time2 = eval { timelocal(0, 0, 0, $d2, $m2-1, $y2) };
+    my $time2 = eval { timelocal(0, 0, 12, $d2, $m2-1, $y2) };
     return unless $time2;	# invalid date
     $time2 -= $datefix if $datefix;
 
@@ -188,7 +209,7 @@ sub parse_date_range {
       sprintf("%04d-%02d-%02d", @tm2) ]
 }
 
-BEGIN { push(@EXPORT, qw(parse_date_range)) }
+push( @EXPORT, qw(parse_date_range) );
 
 sub iso8601date {
     my ($time) = shift || time;
@@ -196,10 +217,66 @@ sub iso8601date {
     sprintf("%04d-%02d-%02d", 1900+$tm[5], 1+$tm[4], $tm[3]);
 }
 
-BEGIN { push(@EXPORT, qw(iso8601date)) }
+push( @EXPORT, qw(iso8601date) );
+
+sub min { $_[0] < $_[1] ? $_[0] : $_[1] }
+sub max { $_[0] > $_[1] ? $_[0] : $_[1] }
+
+push( @EXPORT, qw(min max) );
+
+# Locale / Gettext.
+# Variable expansion. See GNU gettext for details.
+sub __expand($%) {
+    my ($t, %args) = @_;
+    my $re = join('|', map { quotemeta($_) } keys(%args));
+    $t =~ s/\{($re)\}/defined($args{$1}) ? $args{$1} : "{$1}"/ge;
+    $t;
+}
+
+# Translation w/ variables.
+sub __x($@) {
+    my ($t, %vars) = @_;
+    __expand(_T($t), %vars);
+}
+
+# Translation w/ singular/plural handling.
+sub __n($$$) {
+    my ($sing, $plur, $n) = @_;
+    _T($n == 1 ? $sing : $plur);
+}
+
+# Translation w/ singular/plural handling and variables.
+sub __nx($$$@) {
+    my ($sing, $plur, $n, %vars) = @_;
+    __expand(__n($sing, $plur, $n), %vars);
+}
+
+# Make __xn a synonym for __nx.
+*__xn = \&__nx;
+
+# And the dummy...
+sub N__($) { $_[0] };
+
+# This is for context sensitive translations, where e.g., cmd:btw
+# translates to cmd:vat and we deliver need the part after the colon.
+sub __xt {
+    my $t = _T($_[0]);
+    $t =~ s/^.*://;
+    $t;
+}
+
+# Same, without translating.
+# Basically, __xt is __XN(_T($_[0])).
+sub __XN {
+    my $t = $_[0];
+    $t =~ s/^.*://;
+    $t;
+}
+
+push( @EXPORT, qw( __x __n __nx __xn N__ __xt __XN )  );
 
 # ... more to come ...
 
-BEGIN { @EXPORT_OK = @EXPORT }
+@EXPORT_OK = @EXPORT;
 
 1;
