@@ -5,13 +5,16 @@ package EekBoek;
 # NOTE: This is a documentation-only module.
 
 use strict;
+use utf8;
 
 # Minimal version to prevent many Unicode bugs.
 use 5.008003;
 
-my $RCS_Id = '$Id: EekBoek.pm,v 1.73.2.1 2010/01/06 10:53:02 jv Exp $ ';
+require EB::Version;
+
 our $PACKAGE = 'EekBoek';
-our $VERSION = '1.04.06.01';
+
+our $VERSION = $EB::Version::VERSION;
 
 =head1 NAME
 
@@ -44,7 +47,7 @@ Web site: L<http://www.eekboek.nl>.
 
 =head1 COPYRIGHT AND DISCLAIMER
 
-This program is Copyright 2005-2008 by Squirrel Consultancy. All
+This program is Copyright 2005-2011 by Squirrel Consultancy. All
 rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -61,5 +64,3 @@ GNU General Public License or the Artistic License for more details.
 =cut
 
 1;
-
-# $Id: EekBoek.pm,v 1.73.2.1 2010/01/06 10:53:02 jv Exp $ 
